import { MissingEntityError, UninitializedRepositoryError } from '../errors.js';
/**
 * An abstract class for a repository.
 * @typeParam M - Type of the model.
 * @typeParam RQ - Type of possible request.
 * @typeParam RSP - Type of possible response.
 */
export class AbstractRepository {
    constructor(name) {
        this.name = name;
    }
    isInitialized() {
        if (!this.repository)
            return false;
        if (!this.dependenciesMet())
            return false;
        return true;
    }
    throwUninitializedRepositoryError() {
        throw new UninitializedRepositoryError(`${this.name} Repository has not been initialized!`);
    }
    async create(data) {
        if (!this.isInitialized())
            this.throwUninitializedRepositoryError();
        const model = this.repository.create();
        if (data === undefined)
            return model;
        await this.write(model, data);
        return model;
    }
    async save(model) {
        if (!this.isInitialized())
            this.throwUninitializedRepositoryError();
        return await this.repository.save(model);
    }
    async find(options) {
        if (!this.isInitialized())
            this.throwUninitializedRepositoryError();
        const findOptions = {
            ...(options ?? {}),
            relations: options?.relations ?? this.getDefaultFindOptionsRelations(),
        };
        return await this.repository.find(findOptions);
    }
    async findOne(options) {
        if (!this.isInitialized())
            this.throwUninitializedRepositoryError();
        const findOptions = {
            ...options,
            relations: options?.relations ?? this.getDefaultFindOptionsRelations(),
        };
        return await this.repository.findOne(findOptions);
    }
    async findOneOrFail(options) {
        if (!this.isInitialized())
            this.throwUninitializedRepositoryError();
        const findOptions = {
            ...options,
            relations: options.relations ?? this.getDefaultFindOptionsRelations(),
        };
        const model = await this.repository.findOne(findOptions);
        if (!model) {
            throw new MissingEntityError(`The requested ${this.name} does not exist in the database`, 404);
        }
        return model;
    }
    async remove(model) {
        if (!this.isInitialized())
            this.throwUninitializedRepositoryError();
        await this.repository.remove(model);
    }
    getDefaultFindOptionsRelations() {
        return undefined;
    }
    setDependencies(dependencies) {
        this.dependencies = dependencies;
    }
}
//# sourceMappingURL=abstractRepository.js.map